--------------------------------------------------------------------------------
-- Company: 	CIEMAT
-- Engineer:	 Juan de Vicente
--
-- Create Date:    11:11:49 12/04/08
-- Design Name:    
-- Module Name:    DAC_TelAddressConversion - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:  DAC and Telemetry address conversion for the CTBv2 (Clock Transition Board v2)
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity DAC_TelAddressConversion is
    Port ( ADDR_BACKPLANE : in std_logic_vector(7 downto 0);
           ADDR_BACKPLANE_MOD : out std_logic_vector(7 downto 0);
           CS_CFG : in std_logic;
           CS_TEL : in std_logic);
end DAC_TelAddressConversion;

architecture Behavioral of DAC_TelAddressConversion is


	 signal addr_backplane_int: std_logic_vector(7 downto 0);
	 signal addr_dac: std_logic_vector(7 downto 0);
	 signal addr_tel: std_logic_vector(7 downto 0);
	 
	 type rom is array (0 to 91) of std_logic_vector(7 downto 0);

	signal CTBtoCTB2_DACs: rom:=rom'(
	x"40",x"41",x"42",x"43",x"44",x"45",x"0F",x"0F",x"46",x"47",x"58",x"59",x"0F",x"0F",x"0F",x"0F",x"56",x"57",
	x"5A",x"5B",x"4A",x"4B",x"50",x"51",x"48",x"49",x"0F",x"0F",x"0F",x"0F",x"0F",x"0F",x"20",x"21",x"22",x"23",
	x"24",x"25",x"26",x"27",x"28",x"29",x"54",x"55",x"0F",x"0F",x"0F",x"0F",x"30",x"31",x"32",x"33",x"34",x"35",
	x"36",x"37",x"38",x"39",x"0F",x"0F",x"0F",x"0F",x"0F",x"0F",x"00",x"01",x"02",x"03",x"04",x"05",x"08",x"09",
	x"18",x"19",x"14",x"15",x"0F",x"0F",x"0F",x"0F",x"16",x"17",x"52",x"53",x"2A",x"2B",x"10",x"11",x"0A",x"0B",
	x"12",x"13");		  --00 undefined
	
	type rom2 is array (112 to 163) of std_logic_vector(7 downto 0);
 	signal CTBtoCTB2_tel: rom2:=rom2'(
	x"70",x"71",x"72",x"73",x"74",x"75",x"76",x"77",x"78",x"79",x"7A",x"7B",x"7C",x"7D",x"7E",x"7F",x"00",
	x"81",x"A3",x"00",x"A1",x"9D",x"9A",x"87",x"88",x"89",x"9F",x"9B",x"98",x"8D",x"8E",x"8F",x"9C",x"91",
	x"97",x"A2",x"94",x"95",x"9E",x"92",x"8C",x"00",x"86",x"8B",x"90",x"85",x"96",x"8A",x"A0",x"84",x"93",
	x"82");  --00 undefined

begin
  addr_dac <= CTBtoCTB2_DACs(conv_integer(ADDR_BACKPLANE(7 downto 0))); 	
	addr_tel<= CTBtoCTB2_tel(conv_integer(ADDR_BACKPLANE(7 downto 0)));

	addr_backplane_int <= addr_tel when (cs_tel='1') else
	 							 addr_dac when (cs_cfg='1' and ADDR_BACKPLANE < x"5C") else
								 x"FE" when (ADDR_BACKPLANE=x"FE") else
								 x"FF" when (ADDR_BACKPLANE=x"FF") else
								  x"F0";		--undefined


	ADDR_BACKPLANE_MOD <= addr_backplane_int;
end Behavioral;
