library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity guider_CCD is
	Port ( SYSCLK   : in  std_logic;
			 WRT      : in  std_logic;
			 ADDRESS  : in  std_logic_vector (15 downto 0);
			 SEQ_DATA : in  std_logic_vector (11 downto 0);
			 DATA_OUT : out std_logic_vector (47 downto 0);
          V_IN     : in  std_logic_vector (11 downto 0);
          V_OUT    : out std_logic_vector (11 downto 0));
end guider_CCD;

architecture Behavioral of guider_CCD is
	signal ccd1_reg : std_logic_vector (11 downto 0) := x"000";
	signal ccd2_reg : std_logic_vector (11 downto 0) := x"000";
	signal ccd3_reg : std_logic_vector (11 downto 0) := x"000";
	signal load_reg : std_logic_vector (8 downto 0);
	signal clk_temp : std_logic_vector (8 downto 0);
	signal hold     : std_logic_vector (8 downto 0);
	signal q1       : std_logic;
	signal q2       : std_logic;
	signal load     : std_logic;

	type skip_array is array (8 downto 0) of std_logic_vector(11 downto 0);
	signal skip : skip_array;
begin
	guider_CCD_write : process (SYSCLK)
	begin
		if(rising_edge(SYSCLK)) then
			if (WRT = '1') then
				if (ADDRESS = x"0001") then
					ccd1_reg <= SEQ_DATA;
				elsif (ADDRESS = x"0002") then
					ccd2_reg <= SEQ_DATA;
				elsif (ADDRESS = x"0003") then
					ccd3_reg <= SEQ_DATA;
				end if;
			end if;
			q1 <= WRT;
			q2 <= not(q1);
		end if;
	end process guider_CCD_write;

	load     <= q1 and q2 when ADDRESS = x"0004" else '0';
	load_reg <= "000000000";
	hold     <= V_IN(10 downto 8) & V_IN(6 downto 4) & V_IN(2 downto 0) when LOAD = '1' else hold;
	clk_temp <= V_IN(10 downto 8) & V_IN(6 downto 4) & V_IN(2 downto 0) xor hold;

	skip(8) <= ccd3_reg when LOAD = '1' else skip(8) - 1 when falling_edge(clk_temp(8)) = TRUE and skip(8) /= x"000" else skip(8);
	skip(7) <= ccd3_reg when LOAD = '1' else skip(7) - 1 when falling_edge(clk_temp(7)) = TRUE and skip(7) /= x"000" else skip(7);
	skip(6) <= ccd3_reg when LOAD = '1' else skip(6) - 1 when falling_edge(clk_temp(6)) = TRUE and skip(6) /= x"000" else skip(6);
	skip(5) <= ccd2_reg when LOAD = '1' else skip(5) - 1 when falling_edge(clk_temp(5)) = TRUE and skip(5) /= x"000" else skip(5);
	skip(4) <= ccd2_reg when LOAD = '1' else skip(4) - 1 when falling_edge(clk_temp(4)) = TRUE and skip(4) /= x"000" else skip(4);
	skip(3) <= ccd2_reg when LOAD = '1' else skip(3) - 1 when falling_edge(clk_temp(3)) = TRUE and skip(3) /= x"000" else skip(3);
	skip(2) <= ccd1_reg when LOAD = '1' else skip(2) - 1 when falling_edge(clk_temp(2)) = TRUE and skip(2) /= x"000" else skip(2);
	skip(1) <= ccd1_reg when LOAD = '1' else skip(1) - 1 when falling_edge(clk_temp(1)) = TRUE and skip(1) /= x"000" else skip(1);
	skip(0) <= ccd1_reg when LOAD = '1' else skip(0) - 1 when falling_edge(clk_temp(0)) = TRUE and skip(0) /= x"000" else skip(0);

	DATA_OUT <= q1 & q2 & load & load_reg & ccd1_reg & ccd2_reg & ccd3_reg;

	V_OUT(11) <= V_IN(11);
	V_OUT(10) <= V_IN(10) when (skip(8) = x"000" or LOAD = '1') else hold(8);
	V_OUT(9)  <= V_IN(9)  when (skip(7) = x"000" or LOAD = '1') else hold(7);
	V_OUT(8)  <= V_IN(8)  when (skip(6) = x"000" or LOAD = '1') else hold(6);
	V_OUT(7)  <= V_IN(7);
	V_OUT(6)  <= V_IN(6)  when (skip(5) = x"000" or LOAD = '1') else hold(5);
	V_OUT(5)  <= V_IN(5)  when (skip(4) = x"000" or LOAD = '1') else hold(4);
	V_OUT(4)  <= V_IN(4)  when (skip(3) = x"000" or LOAD = '1') else hold(3);
	V_OUT(3)  <= V_IN(3);
	V_OUT(2)  <= V_IN(2)  when (skip(2) = x"000" or LOAD = '1') else hold(2);
	V_OUT(1)  <= V_IN(1)  when (skip(1) = x"000" or LOAD = '1') else hold(1);
	V_OUT(0)  <= V_IN(0)  when (skip(0) = x"000" or LOAD = '1') else hold(0);
end Behavioral;
