--------------------------------------------------------------------------------
-- COUNTER DECLARATION
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity counter is
	Port ( LOAD      : in  std_logic;
          SKIP_DATA : in  std_logic_vector (11 downto 0);
          CLOCK_IN  : in  std_logic;
          CLOCK_OUT : out std_logic;
			 STATUS    : out std_logic);
end counter;

architecture counter_Behavioral of counter is
	signal skip     : std_logic_vector (11 downto 0) := x"000";
	signal clk_temp : std_logic;
	signal hold     : std_logic;
begin

	countdown : process (clk_temp)
	begin
		if falling_edge(clk_temp) then
			if skip /= x"000" then
				skip <= skip - 1;
			end if;
		end if;
		
		if LOAD = '1' then
			skip <= SKIP_DATA;
			hold <= CLOCK_IN;
		end if;
	end process countdown;

	clk_temp  <= CLOCK_IN xor hold;
	CLOCK_OUT <= CLOCK_IN when (skip = x"000" or LOAD = '1') else hold;
	STATUS    <= '0' when skip = x"000" else '1';

end counter_Behavioral;

--------------------------------------------------------------------------------
-- GUIDER CCD CODE
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity guider_CCD is
	Port ( SYSCLK   : in  std_logic;
			 WRT      : in  std_logic;
			 ADDRESS  : in  std_logic_vector (15 downto 0);
			 SEQ_DATA : in  std_logic_vector (11 downto 0);
			 DATA_OUT : out std_logic_vector (47 downto 0);
          V_IN     : in  std_logic_vector (11 downto 0);
          V_OUT    : out std_logic_vector (11 downto 0));
end guider_CCD;

architecture Behavioral of guider_CCD is
	component counter
		Port ( LOAD      : in  std_logic;
				 SKIP_DATA : in  std_logic_vector (11 downto 0);
				 CLOCK_IN  : in  std_logic;
				 CLOCK_OUT : out std_logic;
				 STATUS    : out std_logic);
	end component;

	signal ccd1_reg : std_logic_vector (11 downto 0) := x"000";
	signal ccd2_reg : std_logic_vector (11 downto 0) := x"000";
	signal ccd3_reg : std_logic_vector (11 downto 0) := x"000";
	signal load_reg : std_logic_vector ( 8 downto 0);

	signal q1       : std_logic;
	signal q2       : std_logic;
	signal load     : std_logic;
begin

	guider_CCD_write : process (SYSCLK)
	begin
		if(rising_edge(SYSCLK)) then
			if (WRT = '1') then
				if (ADDRESS = x"0001") then
					ccd1_reg <= SEQ_DATA;
				elsif (ADDRESS = x"0002") then
					ccd2_reg <= SEQ_DATA;
				elsif (ADDRESS = x"0003") then
					ccd3_reg <= SEQ_DATA;
				end if;
			end if;
			q1 <= WRT;
			q2 <= not(q1);
		end if;
	end process guider_CCD_write;

	load     <= q1 and q2 when ADDRESS = x"0004" else '0';
	DATA_OUT <= q1 & q2 & load & load_reg & ccd1_reg & ccd2_reg & ccd3_reg;

	CCD1_V3: counter
		Port map (load, ccd1_reg, V_IN(10), V_OUT(10), load_reg(8));
	CCD1_V2: counter
		Port map (load, ccd1_reg, V_IN(9),  V_OUT(9),  load_reg(7));
	CCD1_V1: counter
		Port map (load, ccd1_reg, V_IN(8),  V_OUT(8),  load_reg(6));
	CCD2_V3: counter
		Port map (load, ccd2_reg, V_IN(6),  V_OUT(6),  load_reg(5));
	CCD2_V2: counter
		Port map (load, ccd2_reg, V_IN(5),  V_OUT(5),  load_reg(4));
	CCD2_V1: counter
		Port map (load, ccd2_reg, V_IN(4),  V_OUT(4),  load_reg(3));
	CCD3_V3: counter
		Port map (load, ccd3_reg, V_IN(2),  V_OUT(2),  load_reg(2));
	CCD3_V2: counter
		Port map (load, ccd3_reg, V_IN(1),  V_OUT(1),  load_reg(1));
	CCD3_V1: counter
		Port map (load, ccd3_reg, V_IN(0),  V_OUT(0),  load_reg(0));

	V_OUT(11) <= V_IN(11);
	V_OUT(7)  <= V_IN(7);
	V_OUT(3)  <= V_IN(3);
end Behavioral;
