-- VHDL model 
--------------------------------------------------------------------------------
-- Company:		   CIEMAT	 
-- Engineer:	   Juan de Vicente
--
-- Create Date:    10:47:16 11/01/06
-- Design Name:    mux_reg
-- Module Name:    mux_reg - Behavioral
-- Project Name:   DES (Monsoon based)
-- Target Device:  
-- Tool versions:  7.1
-- Description:	When the module is selected data from the backplane is used to select clockA and clockB 
-- in the Front Panel with the following format
--****************
-- FRONT_PANEL_REG==DATA_BACKPLANE(15..14)
-- LED_MUX=DATA_BACKPLANE(12)
-- CLOCKB=DATA_BACKPLANE(11 downto 6)
-- CLOCKA=DATA_BACKPLANE(5 downto 0)
--*****************
-- Dependencies:
-- Comments:  Included a translation table (ROM) for mapping to physical address
--  
-- Revision:  
--- Additional Comments:   
-- Revision: 11/16/06- Conversion from mux to realmux for the front panel
-- Revision: 11/07/08- Mux conversion table changed according to clock transion board v2
-- Revision: 03/09/11- When reset leds are disable (bit 13 mux_reg)
--------------------------------------------------------------------------------






library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity mux_reg is
    Port ( SYSCLK : in std_logic;
           RESET : in std_logic;
           WR_BACKPLANE16 : in std_logic;
		   DATA_BACKPLANE : in std_logic_vector(15 downto 0);
		   DATA_OUT_BACKPLANE : out std_logic_vector(15 downto 0);	
           CS_MUX : in std_logic;
		   MUXA        : out   std_logic_vector (3 downto 0); 
       	   MUXB        : out   std_logic_vector (3 downto 0);
		   MUXENA        : out   std_logic_vector (2 downto 0); 
       	   MUXENB        : out   std_logic_vector (2 downto 0);
		   LED_MUX		 : out std_logic
		 );
end mux_reg;

architecture Behavioral of mux_reg is

-- Table for mux front panel clock selection (H2C,H2B,30..0)	
    type rom is array (0 to 63) of std_logic_vector(5 downto 0);
  
 --Old
 --  signal muxtoRmuxA: rom:=rom'(O"07",O"04",O"06",O"33",O"15",O"17",O"16",O"14",O"36",O"23",O"27",O"32",O"01",
--				                 O"03",O"02",O"05",O"12",O"13",O"22",O"25",O"21",O"31",O"30",O"24",O"34",O"35",
--								 O"37",O"42",O"41",O"53",O"26",O"43",O"45");
								 
  --- signal muxtoRmuxB: rom:=rom'(O"07",O"01",O"06",O"34",O"12",O"10",O"11",O"13",O"31",O"22",O"35",O"27",O"04",
--							    O"02",O"03",O"00",O"15",O"14",O"23",O"20",O"24",O"36",O"37",O"21",O"33",O"32",
--							    O"30",O"43",O"44",O"45",O"26",O"42",O"40");
  
	signal muxtoRmuxA: rom:=rom'(
	O"36",O"23",O"27",O"32",O"15",O"17",O"16",O"14",O"07",O"04",O"06",O"33",O"34",O"35",O"37",
	O"42",O"41",O"53",O"22",O"25",O"21",O"31",O"30",O"24",O"01",O"03",O"02",O"05",O"12",O"13",	
	O"45",O"43",O"26",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",
	O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",
	O"00",O"00",O"00",O"00");
							 
  

   signal muxtoRmuxB: rom:=rom'(O"31",O"22",O"35",O"27",
	O"12",O"10",O"11",O"13",
	O"07",O"01",O"06",O"34",
	O"33",O"32",O"30",O"43",O"44",O"45",
	O"23",O"20",O"24",O"36",O"37",O"21",
	O"04",O"02",O"03",O"00",O"15",O"14",
	O"40",O"42",O"26",
	O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",
	O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",O"00",
	O"00",O"00",O"00",O"00");

   signal mux_reg    :std_logic_vector(15 downto 0);

   signal RmuxA	     : std_logic_vector (5 downto 0);
   signal RmuxB	     : std_logic_vector (5 downto 0);	

begin
	
	Signal_Register: process(SYSCLK)
	begin
        if (rising_edge(SYSCLK)) then
            if ( RESET = '1' ) then
                mux_reg <= x"2000";    --(others =>'0');
            elsif (WR_BACKPLANE16 = '1' and CS_MUX = '1') then
		        mux_reg <= DATA_BACKPLANE(15 downto 0);
			end if;
				
		end if;
			
	end process;
		
	 DATA_OUT_BACKPLANE <= mux_reg;	
		
		--FRONT PANEL MUX CONVERSION
	 RmuxA(5 downto 0) <= muxtoRmuxA(conv_integer(mux_reg(5 downto 0))); 
	 RmuxB(5 downto 0) <= muxtoRmuxB(conv_integer(mux_reg(11 downto 6))); 
	  
	  
	   
     MUXA(3 downto 0) <= RmuxA(3 downto 0);
	 MUXENA(0) <= (not RmuxA(5)) and (not RmuxA(4));
	 MUXENA(1) <= (not RmuxA(5)) and (RmuxA(4));
	 MUXENA(2) <= ( RmuxA(5)) and (not RmuxA(4));

	 MUXB(3 downto 0) <= RmuxB(3 downto 0);
	 MUXENB(0) <= (not RmuxB(5)) and (not RmuxB(4));
	 MUXENB(1) <= (not RmuxB(5)) and (RmuxB(4));
	 MUXENB(2) <= ( RmuxB(5)) and (not RmuxB(4));

  	 --END FRONT PANEL MUX CONVERSION
	 LED_MUX <= not mux_reg(12);
			
end Behavioral;
