------------------------------------------------------------------------
-- PROJECT          :   DES	(Monsoon Modification)
-- BOARD            :   IR CLOCK&BIAS BOARD
-- DEVICE ID        :   SPARTAN2 FPGA 
-- PARENT MODULE    :   TOP LEVEL
-- MODULE           :   SEQUENCER (CLOCK) CONTROL MODULE
-- VERSION          :   1.0 
------------------------------------------------------------------------
-- FILE NAME        :   seq_control.vhd
-- AUTHOR           :   Juan de Vicente	(Monsoon G.Rahmer)
-- DATE             :   01 April 2006
-- COMPANY          :   CIEMAT
-- PLATFORM         :   Xilinx ISE Webpack 7.1.04
------------------------------------------------------------------------
-- DESCRIPTION:   
-- This module provides clock signals. This
-- is a write-only access, and the backplane data goes directly to the 
-- CLOCK_DATA(31:0).
--
--
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES:
--
-- The original code (FPGA+CPLD) remains commented in this file
--
--
------------------------------------------------------------------------
-- REVISION HISTORY:
-- DATE       AUTHOR  VERSION   MODIFICATIONS
-- 25/May/03    GR      1.2     Added registration of data and select
-- 16/Feb/04    GR      1.3     Implemented automatic double write for the fast bias data
--                              (required by the parallel DAC AD9709)
-- 10/May/04    GR      1.4     Added bank store of Fast Bias data
-- 01/Dic/05    JU				Integration of seqcpld_control in seq_control
-- 03/April/06	JU      		Bias removed
------------------------------------------------------------------------
-- PORT DEFINITIONS:
--
-- SYSCLK : board clock
-- RESET : reset signal from the backplane interface
-- WR : write signal from the backplane interface
-- ADDR(7:0) : address from the backplane interface
-- CONTROL(15:0) : control register word (not used)
-- DATA_IN: 32 bits. Signals from backplane interface
-- CLOCK_DATA : 32 bits conforming the clock sequence

------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity seq_control is
    Port ( SYSCLK : in std_logic;
           RESET : in std_logic;
           WRITE_32 : in std_logic;
           ADDR : in std_logic_vector(7 downto 0);
           DATA_IN : in std_logic_vector(31 downto 0);
           CLOCK_DATA : out std_logic_vector(31 downto 0)
          );
end seq_control;
	
architecture Behavioral of seq_control is

    signal seqclk_en : std_logic := '0';
	signal data_reg : std_logic_vector(31 downto 0);   
   	signal seq_clk : std_logic;

		

begin
  
            
------------------------------------------------------------------------
-- DATA SEQUENCE OUTPUT PROCESS
------------------------------------------------------------------------
 CLOCK_DATA <= data_reg;
 Signal_Register: process(SYSCLK)
	begin
        if (rising_edge(SYSCLK)) then
            if ( RESET = '1' ) then
                seqclk_en <= '0';
            elsif (WRITE_32 = '1') then
                if (ADDR = "00000") then 
                    data_reg <= (DATA_IN);  --not(DATA_IN);	  --!!!!! INVERTED
                end if;
            end if;
        end if;
    end process;
				
						 

--	Signal_Register: process(SYSCLK)
--	begin
--        if (rising_edge(SYSCLK)) then
--            if ( RESET = '1' ) then
--                seqclk_en <= '0';
--            elsif (WRITE_32 = '1') then
--                if (ADDR = "00000") then 
--                    data_reg <= (DATA_IN);   -- not(DATA_IN);	  --!!!!! INVERTED
--                    seqclk_en <= '1';
--                else
--                    seqclk_en <= '0';
--            	end if;
--            else
--                     seqclk_en <= '0';    
--            end if;
--       end if;
--    end process;
--  
--  seq_clk <= (not SYSCLK) and seqclk_en;
--	clock_patt_register: process (seq_clk)
--	begin
--		if falling_edge (seq_clk) then
--		   		CLOCK_DATA <= data_reg;			   
--		end if;
--	end process;
--

end Behavioral;
